﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml.Serialization;

namespace GameCore
{
    [Serializable]
    public class Map
    {
        [XmlIgnore]
        public GroundTile[,] TileMap { get; set; }

        [XmlElement("TileMap")]
        [EditorBrowsable(EditorBrowsableState.Never)]
        public string[] XmlMap
        {
            get
            {
                var result = new string[TileMap.GetLength(1)];
                int resultIter = 0;
                for (int y = 0; y < TileMap.GetLength(1); y++)
                {
                    string indexes = "";
                    for (int x = 0; x < TileMap.GetLength(0); x++)
                    {
                        indexes += TileMap[x, y].TileIndex.ToString("D2") + ",";
                    }
                    result[resultIter++] = indexes;
                }
                return result;
            }
            set
            {
                string[] testValues = value[0].Split(',', StringSplitOptions.RemoveEmptyEntries);
                TileMap = new GroundTile[testValues.Length, value.Length];
                for (int i = 0; i < value.Length; i++)
                {
                    string[] rowValues = value[i].Split(',', StringSplitOptions.RemoveEmptyEntries);
                    for (int i2 = 0; i2 < rowValues.Length; i2++)
                    {
                        if (int.TryParse(rowValues[i2], out int intValue))
                            TileMap[i2, i] = new GroundTile { TileIndex = intValue, MapX = i2, MapY = i };
                    }
                }
            }
        }
        public int TileWidth { get; set; }
        public int TileHeight { get; set; }
        public List<TrafficPole> Lights { get; set; }
        public List<Building> Buildings { get; set; }
        public List<Car> Cars { get; set; }

        public Map() : this(10, 20)
        {

        }
        public Map(int width, int height)
        {
            TileMap = new GroundTile[width, height];
            TileWidth = 132;
            TileHeight = 78;
            Lights = new List<TrafficPole>();
            Buildings = new List<Building>();
            Cars = new List<Car>();
        }
    }
}
