﻿using GameCore;
using Microsoft.Win32;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Imaging;

namespace MapMaker
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Map _localMap = new Map();

        public MainWindow()
        {
            InitializeComponent();
            InitializeTemplates();
        }

        private void InitializeTemplates()
        {
            //Load Tiles
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_TILES)
            {
                lstTiles.Items.Add(new Image
                {
                    Source = TrafficScene.GetTileImage(template.ImageIndex),
                    Tag = template
                });
            }
            //Load Building
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_BUILDINGS)
            {
                if (template.Type == TemplateType.ModernBuilding || 
                    template.Type == TemplateType.Infrastructure ||
                    template.Type == TemplateType.Nature)
                {
                    lstBuildings.Items.Add(new Image
                    {
                        Source = TrafficScene.GetBuildingImage(template.ImageIndex),
                        Tag = template
                    });
                }
            }
            //Load Cars
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_CARS)
            {
                lstCars.Items.Add(new Image
                {
                    Source = TrafficScene.GetCarImage(template.ImageIndex),
                    Tag = template
                });
            }
            //Load Lights
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_LIGHTS)
            {
                lstLights.Items.Add(new Image
                {
                    Source = TrafficScene.GetLightImage(template.ImageIndex),
                    Tag = template
                });
            }
        }
        private void MnuNewMapClick(object sender, RoutedEventArgs e)
        {
            var dlg = new dlgNewMap
            {
                Owner = this,
                WindowStartupLocation = WindowStartupLocation.CenterOwner
            };
            if (dlg.ShowDialog() == true)
            {
                int baseIndex = 0;
                if (dlg.FillBack == true)
                    baseIndex = dlg.FillBackIndex;

                _localMap = new Map(dlg.MapWidth, dlg.MapHeight);
                for (int x = 0; x < dlg.MapWidth; x++)
                    for (int y = 0; y < dlg.MapHeight; y++)
                        _localMap.TileMap[x, y] = new GroundTile { TileIndex = baseIndex, MapX = x, MapY = y };
                DrawMap();
            }
        }
        private void MnuLoadMapClick(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog
            {
                InitialDirectory = TrafficScene.MAPDIRECTORY,
                Filter = "XML files (.xml)|*.xml"
            };
            if (dlg.ShowDialog() == true)
            {
                //_localMap = TrafficScene.LoadMap(dlg.FileName);
                _localMap = TrafficScene.LoadMapXML(dlg.FileName);
                DrawMap();
            }
        }
        private void MnuSaveMapClick(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog
            {
                InitialDirectory = TrafficScene.MAPDIRECTORY,
                Filter = "XML files (.xml)|*.xml"
            };
            if (dlg.ShowDialog() == true)
                //TrafficScene.SaveMap(dlg.FileName, _localMap);
                TrafficScene.SaveMapXML(dlg.FileName, _localMap);
        }
        private void BtnPreviousTemplateImageClick(object sender, RoutedEventArgs e)
        {
            PreviousTemplate();
        }
        private void BtnNextTemplateImageClick(object sender, RoutedEventArgs e)
        {
            NextTemplate();
        }
        private void LstTilesSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstTiles.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetTileImage(template.ImageIndex));
            }
        }
        private void LstCarsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstCars.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetCarImage(template.ImageIndex));
            }
        }
        private void LstBuildingsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstBuildings.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetBuildingImage(template.ImageIndex));
            }
        }
        private void LstLightsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstLights.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetLightImage(template.ImageIndex));
            }
        }
        private void CnvMapMouseUp(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Right)
            {//Right clicking the map will place the current template block on the map
                AddBlock(e.GetPosition(cnvMap).X, e.GetPosition(cnvMap).Y);
            }
            else if (e.ChangedButton == MouseButton.Left)
            {//left clicking on the map will select a block 
                SelectBlock(e.GetPosition(cnvMap).X, e.GetPosition(cnvMap).Y);
            }
        }
        private void CnvMapMouseMove(object sender, MouseEventArgs e)
        {
            if (e.RightButton == MouseButtonState.Pressed)
                AddBlock(e.GetPosition(cnvMap).X, e.GetPosition(cnvMap).Y);
        }

        private void DrawMap()
        {
            TrafficScene.DrawMap(_localMap, cnvMap, true, true, true);
        }
        private void AddBlock(double x, double y)
        {
            if (imgCurrentTemplate.Tag == null)
                return;
            if (tabBuildBlocks.SelectedIndex == 0)//Tiles Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate tileTemplate)
                {
                    //Barycentric triangle approach to determine if the click is within that tile or anyone "near" it
                    Point pt = TrafficScene.MapCoordsFromScreenPosition(_localMap, x, y);
                    if (pt.X != -1 && pt.Y != -1)
                    {
                        var newImage = TrafficScene.GetTileImage(tileTemplate.ImageIndex);
                        GroundTile theTile = _localMap.TileMap[(int)pt.X, (int)pt.Y];
                        theTile.CurrentImage.Source = newImage;
                        theTile.TileIndex = tileTemplate.ImageIndex;
                    }
                }
            }
            else if (tabBuildBlocks.SelectedIndex == 1)//Buildings Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate buildingTemplate)
                {
                    var newBuilding = new Building { ImageIndex = buildingTemplate.ImageIndex, Opacity = 1.0 };
                    _localMap.Buildings.Add(newBuilding);
                    var img = TrafficScene.GetBuildingImage(buildingTemplate.ImageIndex);
                    int newX = (int)(x - img.Width / 2);
                    int newY = (int)(y - img.Height / 2);
                    if (newX < 0)
                        newX = 0;
                    if (newY < 0)
                        newY = 0;
                    newBuilding.StartXPosition = newX;
                    newBuilding.StartYPosition = newY;
                    DrawMap();
                }
            }
            else if (tabBuildBlocks.SelectedIndex == 2)//cars Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate carTemplate)
                {
                    var newCar = new Car { ImageIndex = carTemplate.ImageIndex, StartXPosition = (int)x, StartYPosition = (int)y, Opacity = 1.0 };
                    _localMap.Cars.Add(newCar);
                    var img = TrafficScene.GetCarImage(carTemplate.ImageIndex);
                    DrawMap();
                }
            }
            else if (tabBuildBlocks.SelectedIndex == 3)//Lights Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate lightTemplate)
                {
                    var newLight = new TrafficPole
                    {
                        ImageIndex = lightTemplate.ImageIndex,
                        StartXPosition = (int)x,
                        StartYPosition = (int)y,
                        Opacity = 1.0
                    };
                    _localMap.Lights.Add(newLight);
                    DrawMap();
                }

            }
        }
        private void SelectBlock(double x, double y)
        {
        }
        private void SetTemplateSelection(TileTemplate template, BitmapImage image)
        {
            txtTemplateBlockName.Text = template.Name;
            imgCurrentTemplate.Tag = template;
            imgCurrentTemplate.Source = image;
        }
        private void SetTemplateImage(TileTemplate template, int index)
        {
            switch (template.Type)
            {
                case TemplateType.Tile:
                    imgCurrentTemplate.Source = TrafficScene.GetTileImage(index);
                    break;
                case TemplateType.Car:
                    imgCurrentTemplate.Source = TrafficScene.GetCarImage(index);
                    break;
                case TemplateType.ModernBuilding:
                case TemplateType.FantasyBuilding:
                case TemplateType.Nature:
                case TemplateType.Props:
                case TemplateType.Stones:
                case TemplateType.Ruins:
                case TemplateType.Infrastructure:
                    imgCurrentTemplate.Source = TrafficScene.GetBuildingImage(index);
                    break;
                case TemplateType.LightFront:
                case TemplateType.LightBack:
                case TemplateType.StaticLight:
                    imgCurrentTemplate.Source = TrafficScene.GetLightImage(index);
                    break;
                default:
                    break;
            }
        }
        private void NextTemplate()
        {
            if (imgCurrentTemplate.Tag is TileTemplate template)
            {
                int newIndex = TrafficScene.GetNextTemplateImageIndex(template);
                SetTemplateImage(template, newIndex);
                template.ImageIndex = newIndex;
            }
        }
        private void PreviousTemplate()
        {
            if (imgCurrentTemplate.Tag is TileTemplate template)
            {
                int newIndex = TrafficScene.GetPreviousTemplateImageIndex(template);
                SetTemplateImage(template, newIndex);
                template.ImageIndex = newIndex;
            }
        }
    }
}
